/*****************************************************************************
**																			**
**	 Name: 	UART  Software Interface										**	
**																			**
******************************************************************************

(C) Copyright 2003 - Analog Devices, Inc.  All rights reserved.

File Name:		main.asm

Date Modified:	04/03/03		BK		Rev 1.0

Software:       GNU DCC 4.1+

Hardware:		BlackfinOne board (BF532)

Special Connections:  
				EIA-232 Serial Cable (1:1)
				Terminal Program (i.e. Hyperterminal for Windows)
				- bit rate doesn't matter
				- 8-bit, no parity, 1 stop bit
				- no handshake
				- echo off

Purpose:		Illustration of UART usage
				- Autobaud Detection using GP Timer 2	 
				- Polling Operation
				- Interrupt Operation
				
05/22/07 Ronetix, ported from ASM to C

******************************************************************************/
#include "../inc/cdefBF532.h"
#include "../inc/uartlib.h"
#include "../inc/sicnames.h"

/*****************************************************************************
 *
 * Implement a little FIFO
 *
 ****************************************************************************/ 		
typedef struct
{
	unsigned char	buff[16];
    unsigned char 	*ppush;            // push pointer
    unsigned char 	*ppop;             // pop pointer
    volatile int 	count;             // number of bytes in FIFO
}
FIFO;

FIFO FiFo;

static char sHead[] = 	"\r\n-------------------------------"
      	    	     	"\r\nADSP-BF533 Blackfin is speaking"
        	    	 	"\r\n(DL =";
static char sTail[] = 	")"
               			"\r\n-------------------------------"
               			"\r\n";
static char sEcho[] = 	"Type any character. The Blackfin UART"
               			"\r\nreturns the echo> ";
               
static char aEchoFifo[16];

static void isr_uart_tx() __attribute__((interrupt_handler));
static void isr_uart_rx() __attribute__((interrupt_handler));
static void isr_uart_error() __attribute__((interrupt_handler));

static int fifo_init( FIFO *ff );
static int fifo_push( FIFO *ff, unsigned char cc );
static int fifo_pop( FIFO *ff );
static int fifo_ready( FIFO *ff );

/*****************************************************************************
 *
 *  Main program starts here!
 *
 ****************************************************************************/ 
int main()
{
	int dd;
	
	dd = uart_autobaud();

/*****************************************************************************
 *
 *  dd holds the timer period value, now.
 *  Apply formula DL = PERIOD / (16 x 8 bits) and call uart_init that writes
 *  the result to the two 8-bit DL registers (DLH:DLL).
 *
 ****************************************************************************/ 
	uart_init( dd>>7 );
	
/*****************************************************************************
 *
 *  Transmit a Hello World string and the content of the DL registers.
 *
 ****************************************************************************/ 
	uart_puts( sHead );
	uart_putreg( dd );
	uart_puts( sTail );
	
/*****************************************************************************
 *
 *  Wait until operation has finished completely. This is optional.
 *
 ****************************************************************************/ 
	uart_wait4temt();
	
/*****************************************************************************
 *
 *  Transmit another string, but use interrupt mode, this time.
 *  First TX interrupt channel must be assigned and enabled.
 *  Assign to EVT9.
 *
 *  Important: if you enable all three UART interrupts channels, it is
 *  recommended to keep this order of priorities:
 *  PRIORITY(Error) >= PRIORITY(RX) >= PRIORITY(TX)
 *
 ****************************************************************************/ 	
/*****************************************************************************
 *
 *  Register TX service routine at EVT9. 
 *
 ****************************************************************************/ 	
	*pEVT9 = isr_uart_tx;

/*****************************************************************************
 *
 *  Unmask EVT9 interrupt.
 *
 ****************************************************************************/ 	
	*pIMASK |= EVT_IVG9;
	
/*****************************************************************************
 *
 *  Enable UART TX interrupt and assign it to EVT9.
 *  Constants used below are defined in "sicnames.h" header.
 *
 ****************************************************************************/ 	
	*pSIC_IAR0 	= 	IVG_SPT0_ERROR(15) | IVG_PPI_ERROR(15)  | IVG_DMA_ERROR(15)  | IVG_PLL_WAKEUP(15) |\
					IVG_RTC(15)        | IVG_UART_ERROR(15) | IVG_SPT1_ERROR(15) | IVG_SPI_ERROR(15);
	
	*pSIC_IAR1 	= 	IVG_SPT1_RX(15) | IVG_SPT0_TX(15) | IVG_SPT0_RX(15) | IVG_PPI(15) |\
					IVG_UART_TX( 9) | IVG_UART_RX(16) | IVG_SPI(15)     | IVG_SPT1_TX(15);	
	
	*pSIC_IAR2 	= 	IVG_PFA(15) | IVG_TIMER2(15) | IVG_TIMER1(15) | IVG_TIMER0(15) |\
					IVG_SWDT(15)| IVG_MEMDMA1(15)| IVG_MEMDMA0(15)| IVG_PFB(15);	
	
	*pSIC_IMASK = IRQ_UART_TX;

/*****************************************************************************
 *
 *  Enable Interrupt Nesting.
 *
 ****************************************************************************/ 	
	asm ( "[--sp] = reti" );	

/*****************************************************************************
 *
 *  Finally enable interrupts inside UART module, by setting proper bits
 *  in the IER register. It is good programming style to clear potential
 *  UART interrupt latches in advance, by reading RBR, LSR and IIR.
 *
 *  Setting the ETBEI bit automatically fires a TX interrupt request. 
 *
 ****************************************************************************/ 	
	*pUART_RBR;
	*pUART_LSR;
	*pUART_IIR;
	*pUART_IER 	= ETBEI;

/*****************************************************************************
 *
 *  Wait until operation has finished completely. Again, this is optional.
 *
 ****************************************************************************/ 
	uart_wait4temt();

/*****************************************************************************
 *
 *  Disable UART TX interrupt again, and enable UART RX interrupt and UART 
 *  Line Error Interrupt.
 *  Simply echo all received characters back to TX. 
 *
 *  Disable nesting during the setup.
 *
 ****************************************************************************/ 		
	asm ( "reti = [sp++]" );	

/*****************************************************************************
 *
 * Init a little FIFO
 *
 ****************************************************************************/ 		
	fifo_init( &FiFo );
	
/*****************************************************************************
 *
 *  Register RX service routine at EVT8 and error routine to EVT7. 
 *
 ****************************************************************************/ 	
	*pEVT7 = isr_uart_error;
	*pEVT8 = isr_uart_rx;

/*****************************************************************************
 *
 *  Mask EVT9 interrupt and unmask EVT7 and EVT8.
 *
 ****************************************************************************/ 	
	*pIMASK |= EVT_IVG7;
	*pIMASK |= EVT_IVG8;
	*pIMASK &= ~EVT_IVG9;

/*****************************************************************************
 *
 *  Enable and assign interrupts.
 *
 ****************************************************************************/ 	
 	*pSIC_IAR0 	= 	IVG_SPT0_ERROR(15) | IVG_PPI_ERROR(15)  | IVG_DMA_ERROR(15)  | IVG_PLL_WAKEUP(15) |\
					IVG_RTC(15)        | IVG_UART_ERROR(15) | IVG_SPT1_ERROR(15) | IVG_SPI_ERROR(15);
	
	*pSIC_IAR1 	= 	IVG_SPT1_RX(15) | IVG_SPT0_TX(15) | IVG_SPT0_RX(15) | IVG_PPI(15) |\
					IVG_UART_TX(15) | IVG_UART_RX(8) | IVG_SPI(15)     | IVG_SPT1_TX(15);	
	
	*pSIC_IAR2 	= 	IVG_PFA(15) | IVG_TIMER2(15) | IVG_TIMER1(15) | IVG_TIMER0(15) |\
					IVG_SWDT(15)| IVG_MEMDMA1(15)| IVG_MEMDMA0(15)| IVG_PFB(15);	
	
	*pSIC_IMASK = IRQ_UART_RX | IRQ_UART_ERROR;

 /*****************************************************************************
 *
 *  Enable Interrupt Nesting.
 *
 ****************************************************************************/ 	
	asm ( "[--sp] = reti" );
	
/*****************************************************************************
 *
 *  Finally enable interrupts inside UART module, by setting proper bits
 *  in the IER register. It is good programming style to clear potential
 *  UART interrupt latches in advance, by reading RBR, LSR and IIR.
 *
 ****************************************************************************/ 		
 	*pUART_RBR;
	*pUART_LSR;
	*pUART_IIR;
	*pUART_IER 	= ELSI | ERBFI;
	
/*****************************************************************************
 *
 *  The following endless loop tests whether data is ready in the aEchoFifo
 *  and trasnmits it, if required. The FIFO is filled by the UART RX ISR.
 *
 ****************************************************************************/ 		
 	while ( 1 )
		if ( fifo_ready( &FiFo ) )
			uart_putc( fifo_pop( &FiFo ) );
			
	return 0;
}

/*****************************************************************************
 *
 *  UART TX Interrupt Service Rouine.
 *
 *  Load next byte from p3 pointer and moves it to THR register until
 *  p3 points to NULL character. Note that a write to THR clears the
 *  TX interrupt request.
 *
 ****************************************************************************/ 
static void isr_uart_tx()
{
	static char *pp = sEcho;
	
	if ( *pp )
	{
		*pUART_THR = *pp++;
		return;
	}
	else
		*pUART_IIR;
}

/*****************************************************************************
 *
 *  UART RX Interrupt Service Rouine.
 *
 *  When new data is received this ISR puts the new data into the aEchoFifo.
 *  Special handling is required for carriage return and backspace.
 *
 *  Note that reading RBR clears the interrupt request.
 *
 ****************************************************************************/ 
static void isr_uart_rx()
{
	char cc;
	
	cc = *pUART_RBR;
	fifo_push( &FiFo, cc );
	
	if ( cc == '\b')
	{
		fifo_push( &FiFo, ' ' );
		fifo_push( &FiFo, '\b' );
	}
	else
	if ( cc == '\r')
		fifo_push( &FiFo, '\n' );
}

/*****************************************************************************
 *
 *  UART Error/Line Status Interrupt Service Rouine.
 *
 *  If an error is reported by the UART Line Status, then this ISR reads
 *  the LSR register to determine the cause of the error (and to clear the
 *  interrupt request).
 *
 *  To indicate the error a 0xFF character will be written to the FIFO. 
 *
 ****************************************************************************/ 
static void isr_uart_error()
{
	*pUART_LSR;
	fifo_push( &FiFo, -1 );
}

/*****************************************************************************
 *
 *  FIFO Init Rouine.
 *
 ****************************************************************************/ 
static int fifo_init( FIFO *ff )
{
    ff->ppush  = ff->buff;
    ff->ppop   = ff->buff;
    ff->count  = 0;

    return 0;
}

/*****************************************************************************
 *
 *  FIFO Push Rouine.
 *
 ****************************************************************************/ 
static int fifo_push( FIFO *ff, unsigned char cc )
{
    if ( ff->count >= sizeof(ff->buff) )
        return -1;

    *ff->ppush = cc;
    ff->ppush++;
  
    ff->count++;

    if (ff->ppush >= (ff->buff+sizeof(ff->buff)))
        ff->ppush = ff->buff;

    return 0;
}

/*****************************************************************************
 *
 *  FIFO Pop Rouine.
 *
 ****************************************************************************/ 
static int fifo_pop( FIFO *ff )
{
	char cc;
	
    cc = *ff->ppop;
	ff->ppop++;
    ff->count--;

    if ( ff->ppop >= (ff->buff+sizeof(ff->buff)) )
        ff->ppop = ff->buff;

    return cc;
}

/*****************************************************************************
 *
 *  FIFO Ready/Count Rouine.
 *
 ****************************************************************************/ 
static int fifo_ready( FIFO *ff )
{
    return ff->count;
}
